#include "def.h"

extern PAGENO treesearch_page(PAGENO PageNo, char *key);
extern int CompareKeys(char *key, char *word);
extern int FreePage(struct PageHdr *PagePtr);
extern struct PageHdr *FetchPage(PAGENO page);
extern void strtolow(char *str);

typedef struct KeyRecord *RecordPtr;
typedef int (*Visitor)(RecordPtr ptr);

/*
 * start from a leaf page and traverse through records
 */
void leaf_page_traverse(const PAGENO leaf_page, Visitor v) {
    PAGENO page = leaf_page;
    int stop = 0;
    while ((!stop) && (page != NULLPAGENO)) {
        struct PageHdr *p = FetchPage(page);
        assert(IsLeaf(p) && "not a leaf page");
        RecordPtr i = p->KeyListPtr;
        int record_count = p->NumKeys;
        while ((!stop) && record_count-- > 0) {
            stop = v(i);
            i = i->Next;
        }
        PAGENO next = p->PgNumOfNxtLfPg;
        FreePage(p);
        page = next;
    }
}

static char *lookup_key;
static char **buf;
static int buf_size;
static int target_size;

int visit(RecordPtr ptr) {
    char *stored_key = ptr->StoredKey;
    int cmp_result = CompareKeys(lookup_key, stored_key);
    if (cmp_result == 1) { /* key > stored */
        buf[buf_size] = (char *) realloc(
            buf[buf_size], sizeof(char) * (strlen(stored_key) + 1));
        strcpy(buf[buf_size], stored_key);
        buf_size++;
    }
    return buf_size == target_size;
}

int get_successors(char *key, int k, char *result[]) {
    strtolow(key);
    const PAGENO page = treesearch_page(ROOT, key);
    buf = result;
    lookup_key = key;
    buf_size = 0;
    target_size = k;
    leaf_page_traverse(page, &visit);
    return buf_size;
}
